@@ -29,8 +29,6 @@ basicConfig(level=config.log_level)
 
 def communicate(frame, event, arg):
     loop = get_loop()
-    if loop.is_running():
-        raise SetFrameError(frame, event, arg)
 
     try:
         loop.run_until_complete(communication_loop(frame, event, arg))
@@ -93,15 +91,26 @@ async def communication_loop(frame_, event_, arg_):
                     "source": file,
                 }
 
-            elif data["type"] == "SET_PROMPT":
+            elif (
+                data["type"] == "SET_PROMPT"
+                or data["type"] == "REFRESH_PROMPT"
+            ):
                 try:
+                    eval_fun = (
+                        serialize_inspect_eval
+                        if data.get("command") == "inspect"
+                        else serialize_diff_eval
+                        if data.get("command") == "diff"
+                        else serialize_answer
+                    )
                     response = {
                         "type": "SET_ANSWER",
                         "key": data["key"],
                         "command": data.get("command"),
                         "frame": data.get("frame"),
-                        **serialize_answer(
-                            data["prompt"], get_frame(frame, data.get("frame"))
+                        **eval_fun(
+                            data["prompt"],
+                            get_frame(frame, data.get("frame")),
                         ),
                     }
                 except SetFrameError as e:
@@ -129,28 +138,6 @@ async def communication_loop(frame_, event_, arg_):
                     **serialize_inspect(data["id"]),
                 }
 
-            elif data["type"] == "REQUEST_INSPECT_EVAL":
-                response = {
-                    "type": "SET_ANSWER",
-                    "key": data["key"],
-                    "command": data.get("command"),
-                    **serialize_inspect_eval(
-                        data["prompt"], get_frame(frame, data.get("frame"))
-                    ),
-                }
-
-            elif data["type"] == "REQUEST_DIFF_EVAL":
-                response = {
-                    "type": "SET_ANSWER",
-                    "key": data["key"],
-                    "command": data.get("command"),
-                    **serialize_diff_eval(
-                        data["left"],
-                        data["right"],
-                        get_frame(frame, data.get("frame")),
-                    ),
-                }
-
             elif data["type"] == "REQUEST_SUGGESTION":
                 response = {
                     "type": "SET_SUGGESTION",
@@ -174,7 +161,8 @@ async def communication_loop(frame_, event_, arg_):
                     stop_trace(frame)
                     die()
                 else:
-                    add_step(command, frame)
+                    step_frame = get_frame(frame, data.get("frame"))
+                    add_step(command, step_frame)
                 stop = True
 
             else:
