@@ -21,7 +21,7 @@ from pprint import pformat
 
 from jedi import Interpreter
 
-from .utils import discompile
+from .utils import cutter_mock, discompile
 from .utils.io import capture_display, capture_exception, capture_std
 from .utils.iterators import iter_cause, iter_stack, iter_frame
 from .utils.obj import (
@@ -29,6 +29,7 @@ from .utils.obj import (
     get_infos,
     obj_cache,
     safe_getattr,
+    safe_repr,
     sync_locals,
 )
 from .errors import SetFrameError
@@ -40,6 +41,7 @@ except ImportError:
     cut = None
 
 log = logging.getLogger(__name__)
+diff_separator = "≏"
 
 
 def get_title(frame, event, arg):
@@ -191,7 +193,7 @@ def serialize_attribute(attr, group):
             attr["signature"] = str(signature(attr["value"]))
         except Exception:
             pass
-    attr["value"] = repr(attr["value"])
+    attr["value"] = safe_repr(attr["value"], "<unrepresentable>", True)
     return attr
 
 
@@ -221,7 +223,8 @@ def serialize_inspect(key):
                     obj,
                     key,
                     f"?! Broken obj: key {key!r} is listed in dir() but "
-                    + f"getattr(obj, {key!r}) raises.",
+                    + f"getattr(obj, {key!r}) raises",
+                    True,
                 ),
             )
             for key in dir(obj)
@@ -259,10 +262,12 @@ def serialize_inspect(key):
         }
     ]
 
-    return {"prompt": repr(obj), "answer": answer}
+    return {"prompt": safe_repr(obj, "<unrepresentable>"), "answer": answer}
 
 
-def serialize_diff_eval(leftStr, rightStr, frame):
+def serialize_diff_eval(prompt, frame):
+    leftStr, rightStr = (s.strip() for s in prompt.split(diff_separator))
+
     try:
         leftKey = get_id_from_expression(leftStr, frame)
     except Exception:
@@ -287,10 +292,10 @@ def serialize_diff_eval(leftStr, rightStr, frame):
             "type": "diff",
             "diff": "".join(
                 difflib.unified_diff(
-                    (pformat(left, indent=2, width=5) + "\n").splitlines(
+                    (pformat(left, indent=2, width=80) + "\n").splitlines(
                         keepends=True
                     ),
-                    (pformat(right, indent=2, width=5) + "\n").splitlines(
+                    (pformat(right, indent=2, width=80) + "\n").splitlines(
                         keepends=True
                     ),
                     fromfile=leftStr,
@@ -300,11 +305,17 @@ def serialize_diff_eval(leftStr, rightStr, frame):
         }
     ]
 
-    return {"prompt": f"{leftStr} ? {rightStr}", "answer": answer}
+    return {
+        "prompt": f"{leftStr} {diff_separator} {rightStr}",
+        "answer": answer,
+    }
 
 
 def serialize_suggestion(prompt, from_, to, cursor, frame):
     answer = {"prompt": prompt, "from": from_, "to": to, "suggestion": {}}
+    # Partial cutter completion support
+    if cut and "!" in prompt:
+        prompt, cursor = cutter_mock(prompt, cursor)
     try:
         script = Interpreter(prompt, [frame.f_locals, frame.f_globals])
         completions = script.complete(cursor["line"] + 1, cursor["ch"])
